'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = Central;
function Central(id, bluetooth, _ref, _ref2) {
  var encode = _ref.encode,
      decode = _ref.decode;
  var SERVICE_UUID = _ref2.SERVICE_UUID,
      CHARACTERISTIC_UUID = _ref2.CHARACTERISTIC_UUID;

  var state = {
    server: null,
    characteristic: null,
    message: '',
    configuration: {
      limit: 20 // HARDCODE: https://devzone.nordicsemi.com/question/3860/largest-allowed-setting-for-variable-length-characteristic/
    },
    id: id
  };

  var isConnected = function isConnected() {
    return state.server && state.server.connected;
  };

  var connect = function connect(name) {
    return bluetooth.requestDevice({
      filters: [{ services: [SERVICE_UUID], name: name }]
    }).then(function (device) {
      state.device = device;
      return device.gatt.connect();
    }).then(function (server) {
      state.server = server;
      return server.getPrimaryService(SERVICE_UUID);
    }).then(function (service) {
      return service.getCharacteristic(CHARACTERISTIC_UUID);
    }).then(function (characteristic) {
      state.characteristic = characteristic;
    });
  };

  var disconnect = function disconnect() {
    if (isConnected()) {
      state.device.gatt.disconnect();
    }
  };

  var listener = function listener(callback) {
    return function (event) {
      var chunk = decode(event.target.value);
      var message = '' + state.message + chunk;
      if (message.startsWith('[[[') && message.endsWith(']]]')) {
        var json = JSON.parse(message.slice(3, message.length - 3));
        callback(json);
        state.message = '';
      } else if (message.startsWith('|||') && message.endsWith('|||')) {
        state.message = '';
      } else {
        state.message = message;
      }
      return state.message;
    };
  };

  var handler = function handler(callback) {
    return state.characteristic.startNotifications().then(function () {
      var eventListener = listener(callback);
      state.characteristic.addEventListener('characteristicvaluechanged', eventListener);
      return state.configuration;
    });
  };

  var write = function write(action) {
    if (!state.server || !state.server.connected || !state.characteristic) return Promise.reject();
    var stringify = '[[[' + JSON.stringify(action) + ']]]';
    var message = encode(stringify);
    var key = encode(state.id + ':');
    var dataSize = state.configuration.limit - key.length;
    var writes = [];

    var i = 0;
    do {
      var next = i + dataSize;
      var end = Math.min(next, message.length);
      var data = message.slice(i, end);
      var buffer = new Uint8Array(key.length + data.length);
      buffer.set(key, 0);
      buffer.set(data, key.length);
      writes.push(buffer);

      i = next;
    } while (i < message.length);

    // Serialize Promises
    return writes.reduce(function (promise, chunk) {
      return promise.then(function () {
        return state.characteristic.writeValue(chunk);
      });
    }, Promise.resolve());
  };

  return {
    connected: isConnected(),
    connect: connect,
    disconnect: disconnect,
    handler: handler,
    write: write,
    listener: listener
  };
}